@@ -1,23 +1,26 @@
 // Copyright 2018-2021 the Deno authors. All rights reserved. MIT license.
 
+use crate::matcher::InnerMatcher;
+use crate::matcher::Matcher;
 use crate::parser::Options;
 use crate::parser::Part;
 use crate::parser::PartModifier;
 use crate::parser::PartType;
 use crate::parser::FULL_WILDCARD_REGEXP_VALUE;
+use crate::regexp::RegExp;
 use crate::tokenizer::is_valid_name_codepoint;
 use crate::Error;
 
 // Ref: https://wicg.github.io/urlpattern/#component
 #[derive(Debug)]
-pub(crate) struct Component {
+pub(crate) struct Component<R: RegExp> {
   pub pattern_string: String,
-  pub rust_regexp: Result<regex::Regex, Error>,
-  pub ecma_regexp_string: String,
+  pub regexp: Result<R, Error>,
   pub group_name_list: Vec<String>,
+  pub matcher: Matcher<R>,
 }
 
-impl Component {
+impl<R: RegExp> Component<R> {
   // Ref: https://wicg.github.io/urlpattern/#compile-a-component
   pub(crate) fn compile<F>(
     input: Option<&str>,
@@ -32,19 +35,17 @@ impl Component {
       &options,
       encoding_callback,
     )?;
-    let (rust_regexp_string, _) =
+    let part_list = part_list.iter().collect::<Vec<_>>();
+    let (regexp_string, name_list) =
       generate_regular_expression_and_name_list(&part_list, &options);
-    let rust_regexp =
-      regex::Regex::new(&rust_regexp_string).map_err(Error::RegEx);
-    let options = options.with_syntax(crate::parser::RegexSyntax::EcmaScript);
-    let (ecma_regexp_string, name_list) =
-      generate_regular_expression_and_name_list(&part_list, &options);
-    let pattern_string = generate_pattern_string(part_list, &options);
+    let regexp = R::parse(&regexp_string).map_err(Error::RegExp);
+    let pattern_string = generate_pattern_string(&part_list, &options);
+    let matcher = generate_matcher::<R>(&part_list, &options);
     Ok(Component {
       pattern_string,
-      rust_regexp,
-      ecma_regexp_string,
+      regexp,
       group_name_list: name_list,
+      matcher,
     })
   }
 
@@ -52,9 +53,9 @@ impl Component {
   pub(crate) fn protocol_component_matches_special_scheme(&self) -> bool {
     const SPECIAL_SCHEMES: [&str; 6] =
       ["ftp", "file", "http", "https", "ws", "wss"];
-    if let Ok(regex) = &self.rust_regexp {
+    if let Ok(regex) = &self.regexp {
       for scheme in SPECIAL_SCHEMES {
-        if regex.captures(scheme).is_some() {
+        if regex.matches(scheme).is_some() {
           return true;
         }
       }
@@ -66,20 +67,15 @@ impl Component {
   pub(crate) fn create_match_result(
     &self,
     input: String,
-    exec_result: regex::Captures,
+    exec_result: Vec<&str>,
   ) -> crate::UrlPatternComponentResult {
-    let mut iter = exec_result.iter();
-    iter.next(); // first match is entire string
-    crate::UrlPatternComponentResult {
-      input,
-      groups: self
-        .group_name_list
-        .clone()
-        .into_iter()
-        .zip(iter.map(|e| e.map(|e| e.as_str().to_string())))
-        .map(|(name, key)| (name, key.unwrap_or_default()))
-        .collect(),
-    }
+    let groups = self
+      .group_name_list
+      .clone()
+      .into_iter()
+      .zip(exec_result.into_iter().map(str::to_owned))
+      .collect();
+    crate::UrlPatternComponentResult { input, groups }
   }
 
   pub(crate) fn optionally_transpose_regex_error(
@@ -87,7 +83,7 @@ impl Component {
     do_transpose: bool,
   ) -> Result<Self, Error> {
     if do_transpose {
-      self.rust_regexp = Ok(self.rust_regexp?);
+      self.regexp = Ok(self.regexp?);
     }
     Ok(self)
   }
@@ -95,7 +91,7 @@ impl Component {
 
 // Ref: https://wicg.github.io/urlpattern/#generate-a-regular-expression-and-name-list
 fn generate_regular_expression_and_name_list(
-  part_list: &[Part],
+  part_list: &[&Part],
   options: &Options,
 ) -> (String, Vec<String>) {
   let mut result = String::from("^");
@@ -163,12 +159,15 @@ fn generate_regular_expression_and_name_list(
 }
 
 // Ref: https://wicg.github.io/urlpattern/#generate-a-pattern-string
-fn generate_pattern_string(part_list: Vec<Part>, options: &Options) -> String {
+fn generate_pattern_string(part_list: &[&Part], options: &Options) -> String {
   let mut result = String::new();
   for (i, part) in part_list.iter().enumerate() {
-    let prev_part: Option<&Part> =
-      if i == 0 { None } else { part_list.get(i - 1) };
-    let next_part: Option<&Part> = part_list.get(i + 1);
+    let prev_part: Option<&Part> = if i == 0 {
+      None
+    } else {
+      part_list.get(i - 1).copied()
+    };
+    let next_part: Option<&Part> = part_list.get(i + 1).copied();
     if part.kind == PartType::FixedText {
       if part.modifier == PartModifier::None {
         result.push_str(&escape_pattern_string(&part.value));
@@ -271,3 +270,87 @@ fn escape_pattern_string(input: &str) -> String {
   }
   result
 }
+
+/// This function generates a matcher for a given parts list.
+fn generate_matcher<R: RegExp>(
+  mut part_list: &[&Part],
+  options: &Options,
+) -> Matcher<R> {
+  fn is_literal(part: &Part) -> bool {
+    part.kind == PartType::FixedText && part.modifier == PartModifier::None
+  }
+
+  // If the first part is a fixed string, we can use it as a literal prefix.
+  let mut prefix = match part_list.first() {
+    Some(part) if is_literal(part) => {
+      part_list = &part_list[1..];
+      part.value.clone()
+    }
+    _ => "".into(),
+  };
+  // If the last part is a fixed string, we can use it as a literal suffix.
+  let mut suffix = match part_list.last() {
+    Some(part) if is_literal(part) => {
+      part_list = &part_list[..part_list.len() - 1];
+      part.value.clone()
+    }
+    _ => "".into(),
+  };
+
+  // If there are no more parts, we must have a prefix and/or a suffix. We can
+  // combine these into a single fixed text literal matcher.
+  if part_list.is_empty() {
+    return Matcher::literal(format!("{prefix}{suffix}"));
+  }
+
+  let inner = match part_list {
+    // If there is only one part, and it is a simple full wildcard with no
+    // prefix or suffix, we can use a simple wildcard matcher.
+    [part]
+      if part.kind == PartType::FullWildcard
+        && part.modifier == PartModifier::None =>
+    {
+      prefix += &part.prefix;
+      if !part.suffix.is_empty() {
+        suffix = format!("{}{suffix}", part.suffix);
+      }
+      InnerMatcher::SingleCapture {
+        filter: None,
+        allow_empty: true,
+      }
+    }
+    // If there is only one part, and it is a simple segment wildcard with no
+    // prefix or suffix, we can use a simple wildcard matcher.
+    [part]
+      if part.kind == PartType::SegmentWildcard
+        && part.modifier == PartModifier::None =>
+    {
+      prefix += &part.prefix;
+      if !part.suffix.is_empty() {
+        suffix = format!("{}{suffix}", part.suffix);
+      }
+      let filter = if options.delimiter_code_point.is_empty() {
+        None
+      } else {
+        Some(options.delimiter_code_point.clone())
+      };
+      InnerMatcher::SingleCapture {
+        filter,
+        allow_empty: false,
+      }
+    }
+    // For all other cases, we fall back to a regexp matcher.
+    part_list => {
+      let (regexp_string, _) =
+        generate_regular_expression_and_name_list(part_list, options);
+      let regexp = R::parse(&regexp_string).map_err(Error::RegExp);
+      InnerMatcher::RegExp { regexp }
+    }
+  };
+
+  Matcher {
+    prefix,
+    suffix,
+    inner,
+  }
+}
