@@ -22,9 +22,9 @@ pub enum RegexSyntax {
 }
 
 // Ref: https://wicg.github.io/urlpattern/#options-header
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 pub struct Options {
-  delimiter_code_point: String, // TODO: It must contain one ASCII code point or the empty string. maybe Option<char>?
+  pub delimiter_code_point: String, // TODO: It must contain one ASCII code point or the empty string. maybe Option<char>?
   pub prefix_code_point: String, // TODO: It must contain one ASCII code point or the empty string. maybe Option<char>?
   regex_syntax: RegexSyntax,
 }
@@ -62,11 +62,6 @@ impl Options {
     }
   }
 
-  pub fn with_syntax(mut self, syntax: RegexSyntax) -> Self {
-    self.regex_syntax = syntax;
-    self
-  }
-
   // Ref: https://wicg.github.io/urlpattern/#escape-a-regexp-string
   pub fn escape_regexp_string(&self, input: &str) -> String {
     assert!(input.is_ascii());
