@@ -5,6 +5,9 @@ use serde::Deserialize;
 use serde::Serialize;
 use url::Url;
 
+use crate::component::Component;
+use crate::parser::RegexSyntax;
+use crate::regexp::RegExp;
 pub use crate::Error;
 
 #[derive(Debug, Default, Clone, PartialEq, Eq, Serialize, Deserialize)]
@@ -47,7 +50,9 @@ pub fn process_construct_pattern_input(
     StringOrInit::String(pattern) => {
       let base_url =
         base_url.map(Url::parse).transpose().map_err(Error::Url)?;
-      crate::UrlPatternInit::parse_constructor_string(&pattern, base_url)?
+      crate::UrlPatternInit::parse_constructor_string::<regex::Regex>(
+        &pattern, base_url,
+      )?
     }
     StringOrInit::Init(init) => {
       if base_url.is_some() {
@@ -90,53 +95,105 @@ pub struct UrlPattern {
 pub struct UrlPatternComponent {
   pub pattern_string: String,
   pub regexp_string: String,
+  pub matcher: Matcher,
   pub group_name_list: Vec<String>,
 }
 
+impl From<Component<EcmaRegexp>> for UrlPatternComponent {
+  fn from(component: Component<EcmaRegexp>) -> Self {
+    Self {
+      pattern_string: component.pattern_string,
+      regexp_string: component.regexp.unwrap().0,
+      matcher: component.matcher.into(),
+      group_name_list: component.group_name_list,
+    }
+  }
+}
+
+#[derive(Debug, Clone, PartialEq, Eq, Serialize, Deserialize)]
+#[serde(rename_all = "camelCase")]
+pub struct Matcher {
+  pub prefix: String,
+  pub suffix: String,
+  #[serde(flatten)]
+  pub inner: InnerMatcher,
+}
+
+#[derive(Debug, Clone, PartialEq, Eq, Serialize, Deserialize)]
+#[serde(rename_all = "camelCase", tag = "kind")]
+pub enum InnerMatcher {
+  Literal {
+    literal: String,
+  },
+  #[serde(rename_all = "camelCase")]
+  SingleCapture {
+    filter: Option<String>,
+    allow_empty: bool,
+  },
+  RegExp {
+    regexp: String,
+  },
+}
+
+impl From<crate::matcher::Matcher<EcmaRegexp>> for Matcher {
+  fn from(matcher: crate::matcher::Matcher<EcmaRegexp>) -> Self {
+    Self {
+      prefix: matcher.prefix,
+      suffix: matcher.suffix,
+      inner: matcher.inner.into(),
+    }
+  }
+}
+
+impl From<crate::matcher::InnerMatcher<EcmaRegexp>> for InnerMatcher {
+  fn from(inner: crate::matcher::InnerMatcher<EcmaRegexp>) -> Self {
+    match inner {
+      crate::matcher::InnerMatcher::Literal { literal } => {
+        Self::Literal { literal }
+      }
+      crate::matcher::InnerMatcher::SingleCapture {
+        filter,
+        allow_empty,
+      } => Self::SingleCapture {
+        filter,
+        allow_empty,
+      },
+      crate::matcher::InnerMatcher::RegExp { regexp } => Self::RegExp {
+        regexp: regexp.unwrap().0,
+      },
+    }
+  }
+}
+
+struct EcmaRegexp(String);
+
+impl RegExp for EcmaRegexp {
+  fn syntax() -> RegexSyntax {
+    RegexSyntax::EcmaScript
+  }
+
+  fn parse(pattern: &str) -> Result<Self, ()> {
+    Ok(EcmaRegexp(pattern.to_string()))
+  }
+
+  fn matches<'a>(&self, text: &'a str) -> Option<Vec<&'a str>> {
+    let regexp = regex::Regex::parse(&self.0).ok()?;
+    regexp.matches(text)
+  }
+}
+
 /// Parse a pattern into its components.
 pub fn parse_pattern(init: crate::UrlPatternInit) -> Result<UrlPattern, Error> {
-  let pattern = crate::UrlPattern::parse_internal(init, false)?;
+  let pattern = crate::UrlPattern::<EcmaRegexp>::parse_internal(init, false)?;
   let urlpattern = UrlPattern {
-    protocol: UrlPatternComponent {
-      pattern_string: pattern.protocol.pattern_string,
-      regexp_string: pattern.protocol.ecma_regexp_string,
-      group_name_list: pattern.protocol.group_name_list,
-    },
-    username: UrlPatternComponent {
-      pattern_string: pattern.username.pattern_string,
-      regexp_string: pattern.username.ecma_regexp_string,
-      group_name_list: pattern.username.group_name_list,
-    },
-    password: UrlPatternComponent {
-      pattern_string: pattern.password.pattern_string,
-      regexp_string: pattern.password.ecma_regexp_string,
-      group_name_list: pattern.password.group_name_list,
-    },
-    hostname: UrlPatternComponent {
-      pattern_string: pattern.hostname.pattern_string,
-      regexp_string: pattern.hostname.ecma_regexp_string,
-      group_name_list: pattern.hostname.group_name_list,
-    },
-    port: UrlPatternComponent {
-      pattern_string: pattern.port.pattern_string,
-      regexp_string: pattern.port.ecma_regexp_string,
-      group_name_list: pattern.port.group_name_list,
-    },
-    pathname: UrlPatternComponent {
-      pattern_string: pattern.pathname.pattern_string,
-      regexp_string: pattern.pathname.ecma_regexp_string,
-      group_name_list: pattern.pathname.group_name_list,
-    },
-    search: UrlPatternComponent {
-      pattern_string: pattern.search.pattern_string,
-      regexp_string: pattern.search.ecma_regexp_string,
-      group_name_list: pattern.search.group_name_list,
-    },
-    hash: UrlPatternComponent {
-      pattern_string: pattern.hash.pattern_string,
-      regexp_string: pattern.hash.ecma_regexp_string,
-      group_name_list: pattern.hash.group_name_list,
-    },
+    protocol: pattern.protocol.into(),
+    username: pattern.username.into(),
+    password: pattern.password.into(),
+    hostname: pattern.hostname.into(),
+    port: pattern.port.into(),
+    pathname: pattern.pathname.into(),
+    search: pattern.search.into(),
+    hash: pattern.hash.into(),
   };
   Ok(urlpattern)
 }
